@@ -1,5 +1,5 @@
 /* audisp-remote.c --
- * Copyright 2008-2011 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2008-2012 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -465,6 +465,8 @@ int main(int argc, char *argv[])
 			CAP_NET_BIND_SERVICE);
 	capng_apply(CAPNG_SELECT_BOTH);
 #endif
+	syslog(LOG_NOTICE, "Audisp-remote started with queue_size: %zu",
+		q_queue_length(queue));
 
 	while (stop == 0) { //FIXME break out when socket is closed
 		fd_set rfd, wfd;
@@ -548,8 +550,11 @@ int main(int argc, char *argv[])
 			} while (remote_fgets_more(sizeof(event)));
 		}
 		// See if output fd is also set
-		if (sock > 0 && FD_ISSET(sock, &wfd)) 
-			send_one(queue);
+		if (sock > 0 && FD_ISSET(sock, &wfd)) {
+			// If so, try to drain backlog
+			while (q_queue_length(queue)&& !suspend && transport_ok)
+				send_one(queue);
+		}
 	}
 	if (sock >= 0) {
 		shutdown(sock, SHUT_RDWR);
@@ -559,7 +564,7 @@ int main(int argc, char *argv[])
 	q_len = q_queue_length(queue);
 	q_close(queue);
 	if (stop)
-		syslog(LOG_NOTICE, "audisp-remote is exiting on stop request");
+		syslog(LOG_NOTICE, "audisp-remote is exiting on stop request, queue_size: %zu", q_len);
 
 	return q_len ? 1 : 0;
 }
@@ -1137,7 +1142,9 @@ static int send_msg_gss (unsigned char *header, const char *msg, uint32_t mlen)
 	utok.value = malloc (utok.length);
 
 	memcpy (utok.value, header, AUDIT_RMW_HEADER_SIZE);
-	memcpy (utok.value+AUDIT_RMW_HEADER_SIZE, msg, mlen);
+	
+	if (msg != NULL && mlen > 0)
+		memcpy (utok.value+AUDIT_RMW_HEADER_SIZE, msg, mlen);
 
 	major_status = gss_wrap (&minor_status,
 				 my_context,
