@@ -1,5 +1,5 @@
 /* auparse.c --
- * Copyright 2006-08 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2006-08,2012 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -70,10 +70,15 @@ static int setup_log_file_array(auparse_state_t *au)
 		num++;
 		snprintf(filename, len, "%s.%d", config.log_file, num);
 	} while (1);
+
+	if (num == 0) {
+		fprintf(stderr, "No log file\n");
+		free_config(&config);
+		return 1;
+	}
 	num--;
 	tmp = malloc((num+2)*sizeof(char *));
 
-
         /* Got it, now process logs from last to first */
 	if (num > 0)
 		snprintf(filename, len, "%s.%d", config.log_file, num);
@@ -189,7 +194,7 @@ auparse_state_t *auparse_init(ausource_t source, const void *b)
 			break;
 		default:
 			errno = EINVAL;
-			return NULL;
+			goto bad_exit;
 			break;
 	}
 	au->source = source;
@@ -216,6 +221,11 @@ bad_exit:
 void auparse_add_callback(auparse_state_t *au, auparse_callback_ptr callback,
 			  void *user_data, user_destroy user_destroy_func)
 {
+	if (au == NULL) {
+		errno = EINVAL;
+		return;
+	}
+
 	if (au->callback_user_data_destroy) {
 		(*au->callback_user_data_destroy)(au->callback_user_data);
 		au->callback_user_data = NULL;
@@ -264,6 +274,15 @@ int auparse_flush_feed(auparse_state_t *au)
 	return 0;
 }
 
+// If there is data in the state machine, return 1
+// Otherwise return 0 to indicate its empty
+int auparse_feed_has_data(const auparse_state_t *au)
+{
+	if (au->parse_state == EVENT_ACCUMULATING)
+		return 1;
+	return 0;
+}
+
 int auparse_reset(auparse_state_t *au)
 {
 	if (au == NULL) {
@@ -680,7 +699,10 @@ static int extract_timestamp(const char *b, au_event_t *e)
 	int rc = 1;
 
         e->host = NULL;
-	tmp = strndupa(b, 80);
+	if (*b == 'n')
+		tmp = strndupa(b, 340);
+	else
+		tmp = strndupa(b, 80);
 	ptr = strtok(tmp, " ");
 	if (ptr) {
 		// Optionally grab the node - may or may not be included
