@@ -1,5 +1,5 @@
 /* typetab.h --
- * Copyright 2007-09,2011 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2007-09,2011-12 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -32,6 +32,8 @@ _S(AUPARSE_TYPE_UID,		"iuid"		)
 _S(AUPARSE_TYPE_UID,		"id"		)
 _S(AUPARSE_TYPE_UID,		"inode_uid"	)
 _S(AUPARSE_TYPE_UID,		"sauid"		)
+_S(AUPARSE_TYPE_UID,		"obj_uid"	)
+_S(AUPARSE_TYPE_GID,		"obj_gid"	)
 _S(AUPARSE_TYPE_GID,		"gid"		)
 _S(AUPARSE_TYPE_GID,		"egid"		)
 _S(AUPARSE_TYPE_GID,		"sgid"		)
@@ -57,6 +59,8 @@ _S(AUPARSE_TYPE_ESCAPED,	"key"		)
 _S(AUPARSE_TYPE_ESCAPED,	"vm"		)
 _S(AUPARSE_TYPE_ESCAPED,	"old-disk"	)
 _S(AUPARSE_TYPE_ESCAPED,	"new-disk"	)
+_S(AUPARSE_TYPE_ESCAPED,	"old-fs"	)
+_S(AUPARSE_TYPE_ESCAPED,	"new-fs"	)
 _S(AUPARSE_TYPE_ESCAPED,	"device"	)
 _S(AUPARSE_TYPE_ESCAPED,	"cgroup"	)
 _S(AUPARSE_TYPE_PERM,		"perm"		)
@@ -72,6 +76,7 @@ _S(AUPARSE_TYPE_SUCCESS,	"result"	)
 _S(AUPARSE_TYPE_A0,		"a0"		)
 _S(AUPARSE_TYPE_A1,		"a1"		)
 _S(AUPARSE_TYPE_A2,		"a2"		)
+_S(AUPARSE_TYPE_A3,		"a3"		)
 _S(AUPARSE_TYPE_SIGNAL,		"sig"		)
 _S(AUPARSE_TYPE_TTY_DATA,	"data"		)
 _S(AUPARSE_TYPE_SESSION,	"ses"		)
@@ -80,7 +85,27 @@ _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pe"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pp"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_fi"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_fp"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"fp"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"fi"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"fe"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"old_pp"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"old_pi"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"old_pe"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"new_pp"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"new_pi"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"new_pe"	)
 _S(AUPARSE_TYPE_NFPROTO,	"family"	)
 _S(AUPARSE_TYPE_ICMPTYPE,	"icmptype"	)
 _S(AUPARSE_TYPE_PROTOCOL,	"proto"		)
 _S(AUPARSE_TYPE_ADDR,		"addr"		)
+#ifdef WITH_APPARMOR
+_S(AUPARSE_TYPE_ESCAPED,	"apparmor"	)
+_S(AUPARSE_TYPE_ESCAPED,	"operation"	)
+_S(AUPARSE_TYPE_ESCAPED,	"denied_mask"	)
+_S(AUPARSE_TYPE_ESCAPED,	"info"		)
+_S(AUPARSE_TYPE_ESCAPED,	"profile"	)
+_S(AUPARSE_TYPE_ESCAPED,	"requested_mask")
+#endif
+_S(AUPARSE_TYPE_PERSONALITY,	"per"		)
+_S(AUPARSE_TYPE_SECCOMP,	"code"		)
+
