@@ -1,5 +1,5 @@
 /* s390x_table.h --
- * Copyright 2005,2006,2008-11 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-06,2008-12 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -301,3 +301,5 @@ _S(336, "open_by_handle_at")
 _S(337, "clock_adjtime")
 _S(338, "syncfs") 
 _S(339, "setns")
+_S(340, "process_vm_readv")
+_S(341, "process_vm_writev")
